<?php

namespace AuthStack\EncryptedLicense\Exception;

class EncryptedLicenseException extends \Exception
{
    public function __construct(...$args)
    {
        parent::__construct(...$args);
    }

    public function render()
    {
        return response()->json(['error' => $this->getMessage()], 500);
    }
}